ExtendedDistributionSpecialization = {}

function ExtendedDistributionSpecialization:GenerateRandomHash()
    local charset = "abcdefghijklmnopqrstuvwxyz0123456789"
    local hash = ""

    for i = 1, 10 do
        local randomIndex = math.random(1, #charset)
        hash = hash .. charset:sub(randomIndex, randomIndex)
    end

    return hash
end

function ExtendedDistributionSpecialization:ToStringData(DistributionList)
    local HashesString = "";
    local FillTypesString = "";

    for fillType, hash in pairs(DistributionList) do
        local Delimiter = "/"
        if #HashesString == 0 then Delimiter = "" end

        HashesString = HashesString .. Delimiter .. hash;
        FillTypesString = FillTypesString .. Delimiter .. fillType;
    end

    return HashesString,FillTypesString
end

function ExtendedDistributionSpecialization:FromStringData(HashesString, FillTypesString)
    
    local Delimiter = "/"

    local FillTypes = {}
    local Hashes = {}

    for part in string.gmatch(FillTypesString, "([^" .. Delimiter .. "]+)") do
        table.insert(FillTypes, part)
    end

    for part in string.gmatch(HashesString, "([^" .. Delimiter .. "]+)") do
        table.insert(Hashes, part)
    end

    local DistributionList = {}

    for i, fillType in ipairs(FillTypes) do
        local NewHash = Hashes[i];
        if NewHash == "-1" then NewHash = -1 end

        DistributionList[tonumber(fillType)] = NewHash;
    end

    return DistributionList;
end

function ExtendedDistributionSpecialization.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(PlaceableProductionPoint, specializations)
end

function ExtendedDistributionSpecialization.registerEventListeners(placeableType)
	SpecializationUtil.registerEventListener(placeableType, "onLoad", ExtendedDistributionSpecialization)
    SpecializationUtil.registerEventListener(placeableType, "onReadStream", ExtendedDistributionSpecialization)
    SpecializationUtil.registerEventListener(placeableType, "onWriteStream", ExtendedDistributionSpecialization)
	SpecializationUtil.registerEventListener(placeableType, "onReadUpdateStream", ExtendedDistributionSpecialization)
	SpecializationUtil.registerEventListener(placeableType, "onWriteUpdateStream", ExtendedDistributionSpecialization)
end

function ExtendedDistributionSpecialization:onLoad(savegame)
    self.UniqueHash = ExtendedDistributionSpecialization:GenerateRandomHash();

    self.spec_productionPoint.DistributionList = {}

    for i = 1, #self.spec_productionPoint.productionPoint.outputFillTypeIdsArray do
        local fillType = self.spec_productionPoint.productionPoint.outputFillTypeIdsArray[i]
        self.spec_productionPoint.DistributionList[fillType] = -1;
    end
end

function ExtendedDistributionSpecialization.registerSavegameXMLPaths(schema, basePath)
    schema:register(XMLValueType.STRING, "#UniqueHash", "UniqueHash")
    schema:register(XMLValueType.STRING, "#FillTypes", "FillTypes")
    schema:register(XMLValueType.STRING, "#Hashes", "Hashes")
end

function ExtendedDistributionSpecialization:saveToXMLFile(xmlFile, key, usedModNames)
    xmlFile:setString(key .. "#UniqueHash", self.UniqueHash)

    local HashesString, FillTypesString = ExtendedDistributionSpecialization:ToStringData(self.spec_productionPoint.DistributionList);
    xmlFile:setString(key .. "#FillTypes", FillTypesString)
    xmlFile:setString(key .. "#Hashes", HashesString)
end

function ExtendedDistributionSpecialization:loadFromXMLFile(xmlFile, key)
    local Hash = xmlFile:getString(key .. "#UniqueHash") 
    local FillTypes = xmlFile:getString(key .. "#FillTypes")
    local Hashes = xmlFile:getString(key .. "#Hashes")

    if Hash ~= nil and FillTypes ~= nil and Hashes ~= nil then
        self.UniqueHash = Hash
        self.spec_productionPoint.DistributionList = ExtendedDistributionSpecialization:FromStringData(Hashes, FillTypes);    
    end
end

function ExtendedDistributionSpecialization:onReadStream(streamId, connection)
    self.spec_husbandryAnimals.UniqueHash = streamReadString(streamId)

    local FillTypes = streamReadString(streamId)
    local Hashes = streamReadString(streamId)

    self.spec_productionPoint.DistributionList = ExtendedDistributionSpecialization:FromStringData(Hashes, FillTypes);
end

function ExtendedDistributionSpecialization:onWriteStream(streamId, connection)
    local HashesString, FillTypesString = ExtendedDistributionSpecialization:ToStringData(self.spec_productionPoint.DistributionList);

    streamWriteString(streamId, self.spec_husbandryAnimals.UniqueHash)

    streamWriteString(streamId, FillTypesString)
    streamWriteString(streamId, FillTypesString)
end

function ExtendedDistributionSpecialization:onReadUpdateStream(streamId, connection)
    self.spec_husbandryAnimals.UniqueHash = streamReadString(streamId)

    local FillTypes = streamReadString(streamId)
    local Hashes = streamReadString(streamId)

    self.spec_productionPoint.DistributionList = ExtendedDistributionSpecialization:FromStringData(Hashes, FillTypes);
end

function ExtendedDistributionSpecialization:onWriteUpdateStream(streamId, connection)
    local HashesString, FillTypesString = ExtendedDistributionSpecialization:ToStringData(self.spec_productionPoint.DistributionList);

    streamWriteString(streamId, self.spec_husbandryAnimals.UniqueHash)

    streamWriteString(streamId, FillTypesString)
    streamWriteString(streamId, FillTypesString)
end