ExtendedProductionFrame = {};

function ExtendedProductionFrame:updateMenuButtons()
    local _, productionPoint = self:getSelectedProduction()
    local isProductionListActive = self.productionList == FocusManager:getFocusedElement()
    local fillType, isInput = self:getSelectedStorageFillType()
    local outputMode = productionPoint:getOutputDistributionMode(fillType)

    local HasTarget = productionPoint.owningPlaceable.spec_productionPoint.DistributionList[fillType] ~= -1;

    if not isProductionListActive and not isInput and outputMode == ProductionPoint.OUTPUT_MODE.AUTO_DELIVER and not HasTarget then
        table.insert(self.menuButtonInfo, {
			profile = "buttonOk",
			inputAction = InputAction.MENU_EXTRA_1,
			text = self.i18n:getText("button_select"),
			callback = function()
                ExtendedProductionFrame:SelectTargetButtonCallback(fillType, productionPoint, self);
			end
		})

		self:setMenuButtonInfoDirty()
    end
end

function ExtendedProductionFrame:populateCellForItemInSection(list, section, index, cell)
    if list == self.productionList then return; end
    if section == 1 then return; end

    local _, productionPoint = self:getSelectedProduction()
    local fillType = self.selectedProductionPoint.outputFillTypeIdsArray[index]
    local outputMode = productionPoint:getOutputDistributionMode(fillType)

    local Target = productionPoint.owningPlaceable.spec_productionPoint.DistributionList[fillType];

    if outputMode == ProductionPoint.OUTPUT_MODE.AUTO_DELIVER and Target ~= -1 then
        cell:getAttribute("outputMode"):setText(self.i18n:getText("ui_production_output_distributing")..": "..SelectDistributionTarget:GetNameByHash(Target))
    end
end

function ExtendedProductionFrame:onButtonToggleOutputMode()
    local _, productionPoint = self:getSelectedProduction()
    local fillType = self:getSelectedStorageFillType()

    productionPoint.owningPlaceable.spec_productionPoint.DistributionList[fillType] = -1;
end

function ExtendedProductionFrame:SelectTargetButtonCallback(fillType, productionPoint, ProductionFrameInstance)
    local Hashes, Names = SelectDistributionTarget:GetHashsNamesByFilltypeInput(fillType, productionPoint.owningPlaceable:getOwnerFarmId(), productionPoint.owningPlaceable.UniqueHash)

    if #Names == 0 then
        g_gui:showInfoDialog({
            visible = true,
            text = g_i18n:getText("text_NoProductDestination"),
            dialogType = DialogElement.TYPE_INFO
        })
    else
        g_gui:showOptionDialog({
            title = g_fillTypeManager:getFillTypeByIndex(fillType).title,
            text = g_i18n:getText("text_ProductDestination"),
            options = Names,
            callback = function(selected)
                if selected == 0 then return end
                productionPoint.owningPlaceable.spec_productionPoint.DistributionList[fillType] = Hashes[selected];
                ProductionFrameInstance.storageList:reloadData()
            end
        })
    end
end

InGameMenuProductionFrame.populateCellForItemInSection = Utils.appendedFunction(InGameMenuProductionFrame.populateCellForItemInSection, ExtendedProductionFrame.populateCellForItemInSection);
InGameMenuProductionFrame.onButtonToggleOutputMode = Utils.prependedFunction(InGameMenuProductionFrame.onButtonToggleOutputMode, ExtendedProductionFrame.onButtonToggleOutputMode);
InGameMenuProductionFrame.updateMenuButtons = Utils.appendedFunction(InGameMenuProductionFrame.updateMenuButtons, ExtendedProductionFrame.updateMenuButtons);

addModEventListener(ExtendedProductionFrame);
