SelectDistributionTarget = {};

function SelectDistributionTarget:InsertSpecialization()
    if g_placeableSpecializationManager:getSpecializationByName("ExtendedDistributionSpecialization") == nil then
        g_placeableSpecializationManager:addSpecialization("ExtendedDistributionSpecialization", "ExtendedDistributionSpecialization", Utils.getFilename("ExtendedDistributionSpecialization.lua", g_currentModDirectory), nil)
    end

    TypeManager.validateTypes = Utils.appendedFunction(TypeManager.validateTypes, function(self)
        if self.typeName == "placeable" then
            for placeableType, placeable in pairs(g_placeableTypeManager:getTypes()) do
                if SpecializationUtil.hasSpecialization(PlaceableProductionPoint, placeable.specializations) then
                    g_placeableTypeManager:addSpecialization(placeableType, "ExtendedDistributionSpecialization")
                end
            end
        end
    end)
end

function SelectDistributionTarget:loadMap()
    addConsoleCommand("DistributeGoodsTest", "DistributeGoodsTest", "DistributeGoodsTest", self)
end

function SelectDistributionTarget:DistributeGoodsTest()
    print("Distribute Goods Test");
    g_currentMission.productionChainManager:distributeGoods();
end

function SelectDistributionTarget:GetHashsNamesByFilltypeInput(fillType, farmid, selfexclude)
    local Hashes = {}
    local Names = {}

    for i = #g_currentMission.placeableSystem.placeables, 1, -1 do
        local CurrentPlaceable = g_currentMission.placeableSystem.placeables[i]
        if CurrentPlaceable.spec_productionPoint ~= nil and farmid == CurrentPlaceable:getOwnerFarmId() then
            for z = 1, #CurrentPlaceable.spec_productionPoint.productionPoint.inputFillTypeIdsArray do
                if CurrentPlaceable.spec_productionPoint.productionPoint.inputFillTypeIdsArray[z] == fillType then
                    if selfexclude ~= CurrentPlaceable.UniqueHash then
                    table.insert(Names, CurrentPlaceable:getName())
                    table.insert(Hashes, CurrentPlaceable.UniqueHash)
                    end
                end
            end
        end
    end

    return Hashes, Names
end

function SelectDistributionTarget:GetNameByHash(Hash)

    for i = #g_currentMission.placeableSystem.placeables, 1, -1 do
        local CurrentPlaceable = g_currentMission.placeableSystem.placeables[i]
        if CurrentPlaceable.spec_productionPoint ~= nil and CurrentPlaceable.UniqueHash == Hash then
            return CurrentPlaceable:getName();
        end
    end

    return "Unknown"
end

function SelectDistributionTarget:GetPlaceableByHash(Hash)

    for i = #g_currentMission.placeableSystem.placeables, 1, -1 do
        local CurrentPlaceable = g_currentMission.placeableSystem.placeables[i]
        if CurrentPlaceable.spec_productionPoint ~= nil and CurrentPlaceable.UniqueHash == Hash then
            return CurrentPlaceable;
        end
    end

    return Hashes, Names
end

function SelectDistributionTarget:RemovePlaceable(placeable)
    local RemovedHash = placeable.UniqueHash;
    for i = #g_currentMission.placeableSystem.placeables, 1, -1 do
        local CurrentPlaceable = g_currentMission.placeableSystem.placeables[i]
        if CurrentPlaceable.spec_productionPoint ~= nil then
            for fillType, hash in pairs(CurrentPlaceable.spec_productionPoint.DistributionList) do
                if hash == RemovedHash then
                    CurrentPlaceable.spec_productionPoint.DistributionList[fillType] = -1 
                end;
            end
        end
    end
end

function SelectDistributionTarget:DistributeGoods(super, ...)

    for _, farmTable in pairs(self.farmIds) do
        for i = 1, #farmTable.productionPoints do
            local distributingProdPoint = farmTable.productionPoints[i]
            local DistributionList = distributingProdPoint.owningPlaceable.spec_productionPoint.DistributionList
            for i = 1, #distributingProdPoint.owningPlaceable.spec_productionPoint.productionPoint.outputFillTypeIdsArray do
                local fillType = distributingProdPoint.owningPlaceable.spec_productionPoint.productionPoint.outputFillTypeIdsArray[i]                
                if DistributionList[fillType] ~= -1 then
                    local ReciverPlaceable = SelectDistributionTarget:GetPlaceableByHash(DistributionList[fillType])

                    local AmountToDistribute = distributingProdPoint.storage:getFillLevel(fillType)
                    local MaxRecive = ReciverPlaceable.spec_productionPoint.productionPoint.storage:getFreeCapacity(fillType, true)
                    
                    if AmountToDistribute > 0 and MaxRecive > 0 then
                        local Transfer = math.min(MaxRecive, AmountToDistribute)
                        distributingProdPoint.storage:setFillLevel(AmountToDistribute - Transfer, fillType)
                        ReciverPlaceable.spec_productionPoint.productionPoint.storage:setFillLevel(ReciverPlaceable.spec_productionPoint.productionPoint.storage:getFillLevel(fillType) + Transfer, fillType)
                    end
                end
            end
        end
    end

    SelectDistributionTarget.StorageBlocks = {};

    for _, farmTable in pairs(self.farmIds) do
        for i = 1, #farmTable.productionPoints do
            local distributingProdPoint = farmTable.productionPoints[i]
            local DistributionList = distributingProdPoint.owningPlaceable.spec_productionPoint.DistributionList
            
            for i = 1, #distributingProdPoint.owningPlaceable.spec_productionPoint.productionPoint.outputFillTypeIdsArray do
                local fillType = distributingProdPoint.owningPlaceable.spec_productionPoint.productionPoint.outputFillTypeIdsArray[i]
                
                if DistributionList[fillType] ~= -1 then
                    table.insert(SelectDistributionTarget.StorageBlocks, {Object = distributingProdPoint.storage, fillType = fillType})
                end
            end
        end
    end

    super(self, ...)

    SelectDistributionTarget.StorageBlocks = nil
end

function SelectDistributionTarget:GetFillLevel(super, fillType)
    if SelectDistributionTarget.StorageBlocks == nil then 
        return super(self, fillType)
    end

    for _, Append in pairs(SelectDistributionTarget.StorageBlocks) do
        if Append.Object == self and Append.fillType == fillType then
            --print("zero"..fillType)
            return 0             
        end
    end

    return super(self, fillType)
end

ProductionChainManager.distributeGoods = Utils.overwrittenFunction(ProductionChainManager.distributeGoods, SelectDistributionTarget.DistributeGoods)
PlaceableSystem.removePlaceable = Utils.prependedFunction(PlaceableSystem.removePlaceable, SelectDistributionTarget.RemovePlaceable);
Storage.getFillLevel = Utils.overwrittenFunction(Storage.getFillLevel, SelectDistributionTarget.GetFillLevel)

SelectDistributionTarget:InsertSpecialization();
addModEventListener(SelectDistributionTarget);